@@ -35,6 +35,7 @@
 #include <KLocale>
 #include <KToggleAction>
 #include <KXMLGUIFactory>
+#include <QStringList>
 
 // Konsole
 #include "ColorScheme.h"
@@ -45,6 +46,8 @@
 #include "SessionManager.h"
 #include "ViewContainer.h"
 #include "ViewSplitter.h"
+#include "konsoleadaptor.h"
+#include "Profile.h"
 
 using namespace Konsole;
 
@@ -58,7 +61,7 @@ ViewManager::ViewManager(QObject* parent , KActionCollection* collection)
 {
     // create main view area
     _viewSplitter = new ViewSplitter(0);  
-	KAcceleratorManager::setNoAccel(_viewSplitter);
+    KAcceleratorManager::setNoAccel(_viewSplitter);
 
     // the ViewSplitter class supports both recursive and non-recursive splitting,
     // in non-recursive mode, all containers are inserted into the same top-level splitter
@@ -85,6 +88,11 @@ ViewManager::ViewManager(QObject* parent , KActionCollection* collection)
             SLOT(profileChanged(Profile::Ptr)) );
     connect( SessionManager::instance() , SIGNAL(sessionUpdated(Session*)) , this,
             SLOT(updateViewsForSession(Session*)) );
+
+    //prepare DBus communication
+    new KonsoleAdaptor(this);
+    QDBusConnection::sessionBus().registerObject(QLatin1String("/Konsole"), this);
+
 }
 
 ViewManager::~ViewManager()
@@ -128,10 +136,10 @@ void ViewManager::setupActions()
 
     KAction* nextViewAction = new KAction( i18n("Next View") , this );
     KAction* previousViewAction = new KAction( i18n("Previous View") , this );
-    QAction* nextContainerAction = new QAction( i18n("Next View Container") , this);
+    KAction* nextContainerAction = new KAction( i18n("Next View Container") , this);
   
-    QAction* moveViewLeftAction = new QAction( i18n("Move View Left") , this );
-    QAction* moveViewRightAction = new QAction( i18n("Move View Right") , this );
+    KAction* moveViewLeftAction = new KAction( i18n("Move View Left") , this );
+    KAction* moveViewRightAction = new KAction( i18n("Move View Right") , this );
 
     // list of actions that should only be enabled when there are multiple view
     // containers open
@@ -177,7 +185,7 @@ void ViewManager::setupActions()
         // to Ctrl+D - which will terminate the session in many cases
         detachViewAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_H) );
        
-  		connect( this , SIGNAL(splitViewToggle(bool)) , this , SLOT(updateDetachViewState()) ); 
+          connect( this , SIGNAL(splitViewToggle(bool)) , this , SLOT(updateDetachViewState()) ); 
         connect( detachViewAction , SIGNAL(triggered()) , this , SLOT(detachActiveView()) );
    
         // Expand & Shrink Active View
@@ -201,6 +209,19 @@ void ViewManager::setupActions()
         collection->addAction("next-container",nextContainerAction);
         collection->addAction("move-view-left",moveViewLeftAction);
         collection->addAction("move-view-right",moveViewRightAction);
+
+        // Switch to tab N shortcuts
+        const int SWITCH_TO_TAB_COUNT = 10;
+        QSignalMapper* switchToTabMapper = new QSignalMapper(this);
+        connect(switchToTabMapper,SIGNAL(mapped(int)),this,SLOT(switchToView(int)));
+        for (int i=0;i < SWITCH_TO_TAB_COUNT;i++)
+        {
+            KAction* switchToTabAction = new KAction(i18n("Switch to Tab %1",i+1),this);
+            switchToTabMapper->setMapping(switchToTabAction,i);
+            connect(switchToTabAction,SIGNAL(triggered()),switchToTabMapper,
+                    SLOT(map()));
+            collection->addAction(QString("switch-to-tab-%1").arg(i),switchToTabAction);
+        }
     }
 
     QListIterator<QAction*> iter(multiViewOnlyActions);
@@ -235,20 +256,29 @@ void ViewManager::setupActions()
     connect( moveViewRightAction , SIGNAL(triggered()) , this , SLOT(moveActiveViewRight()) );
     _viewSplitter->addAction(moveViewRightAction);
 }
-
+void ViewManager::switchToView(int index)
+{
+    Q_ASSERT(index >= 0);
+    ViewContainer* container = _viewSplitter->activeContainer();
+    Q_ASSERT( container );
+    QList<QWidget*> containerViews = container->views();
+    if (index >= containerViews.count())
+        return;
+    container->setActiveView(containerViews.at(index));
+}
 void ViewManager::updateDetachViewState()
 {
-	if (!_actionCollection)
-		return;
+    if (!_actionCollection)
+        return;
 
 
-	bool splitView = _viewSplitter->containers().count() >= 2;
-	bool shouldEnable = splitView || _viewSplitter->activeContainer()->views().count() >= 2;
+    bool splitView = _viewSplitter->containers().count() >= 2;
+    bool shouldEnable = splitView || _viewSplitter->activeContainer()->views().count() >= 2;
 
-	QAction* detachAction = _actionCollection->action("detach-view");
+    QAction* detachAction = _actionCollection->action("detach-view");
 
-	if ( detachAction && shouldEnable != detachAction->isEnabled() )
-		detachAction->setEnabled(shouldEnable);
+    if ( detachAction && shouldEnable != detachAction->isEnabled() )
+        detachAction->setEnabled(shouldEnable);
 }
 void ViewManager::moveActiveViewLeft()
 {
@@ -314,19 +344,19 @@ void ViewManager::detachActiveView()
 
 void ViewManager::sessionFinished()
 {
-	// if this slot is called after the view manager's main widget
-	// has been destroyed, do nothing
-	if (!_viewSplitter)
-		return;
+    // if this slot is called after the view manager's main widget
+    // has been destroyed, do nothing
+    if (!_viewSplitter)
+        return;
 
     Session* session = qobject_cast<Session*>(sender());
 
     if ( _sessionMap[qobject_cast<TerminalDisplay*>(activeView())] == session )
     {
-        // switch to the previous view before deleting the session views to prevent flicker 
+        // switch to the next view before deleting the session views to prevent flicker 
         // occurring as a result of an interval between removing the active view and switching
-        // to the previous view
-        previousView();
+        // to the next view
+        nextView();
     }
 
     Q_ASSERT(session);
@@ -433,7 +463,7 @@ void ViewManager::removeContainer(ViewContainer* container)
         _sessionMap.remove(display);
     } 
 
-	_viewSplitter->removeContainer(container);
+    _viewSplitter->removeContainer(container);
     container->deleteLater();
 
     emit splitViewToggle( _viewSplitter->containers().count() > 1 );
@@ -482,35 +512,41 @@ SessionController* ViewManager::createController(Session* session , TerminalDisp
     connect( session , SIGNAL(destroyed()) , controller , SLOT(deleteLater()) );
     connect( view , SIGNAL(destroyed()) , controller , SLOT(deleteLater()) );
 
-	// if this is the first controller created then set it as the active controller
-	if (!_pluggedController)
-		controllerChanged(controller);
+    // if this is the first controller created then set it as the active controller
+    if (!_pluggedController)
+        controllerChanged(controller);
 
     return controller;
 }
 
 void ViewManager::controllerChanged(SessionController* controller)
 {
-	if ( controller == _pluggedController )
-		return;
+    if ( controller == _pluggedController )
+        return;
 
-	_viewSplitter->setFocusProxy(controller->view());
+    _viewSplitter->setFocusProxy(controller->view());
 
-	_pluggedController = controller;
-	emit activeViewChanged(controller);
+    _pluggedController = controller;
+    emit activeViewChanged(controller);
 }
 
 SessionController* ViewManager::activeViewController() const
 {
-	return _pluggedController;
+    return _pluggedController;
 }
+
+IncrementalSearchBar* ViewManager::searchBar() const
+{
+    return _viewSplitter->activeSplitter()->activeContainer()->searchBar();
+}
+
 void ViewManager::createView(Session* session, ViewContainer* container, int index)
 {
-	// notify this view manager when the session finishes so that its view
+    // notify this view manager when the session finishes so that its view
     // can be deleted
-	//
-	// TODO - Find a more efficient a way to avoid multiple connections
-	disconnect( session , SIGNAL(finished()) , this , SLOT(sessionFinished()) );
+    //
+    // TODO - Find a more efficient a way to avoid multiple connections
+    disconnect( session , SIGNAL(finished()) , this , SLOT(sessionFinished()) );
     connect( session , SIGNAL(finished()) , this , SLOT(sessionFinished()) );
 
      bool isFirst = _sessionMap.isEmpty();
@@ -527,7 +563,7 @@ void ViewManager::createView(Session* session, ViewContainer* container, int ind
      session->addView(display);
 
      // tell the session whether it has a light or dark background
- 	const Profile::Ptr profile = SessionManager::instance()->sessionProfile(session);
+     const Profile::Ptr profile = SessionManager::instance()->sessionProfile(session);
      session->setDarkBackground( colorSchemeForProfile(profile)->hasDarkBackground() );
 
      if ( container == _viewSplitter->activeContainer() ) 
@@ -535,8 +571,8 @@ void ViewManager::createView(Session* session, ViewContainer* container, int ind
          container->setActiveView(display);
          display->setFocus( Qt::OtherFocusReason );
      }
-	
-	 updateDetachViewState();
+    
+     updateDetachViewState();
 }
 
 void ViewManager::createView(Session* session)
@@ -558,7 +594,7 @@ void ViewManager::createView(Session* session)
     while ( containerIter.hasNext() )
     {
         ViewContainer* container = containerIter.next();
-		createView(session,container,-1);
+        createView(session,container,-1);
     }
 
 }
@@ -578,7 +614,7 @@ ViewContainer* ViewManager::createContainer(const Profile::Ptr info)
     switch ( _navigationMethod )
     {
         case TabbedNavigation:    
-            container = new TabbedViewContainerV2(position,_viewSplitter);
+            container = new TabbedViewContainer(position,_viewSplitter);
             break;
         case NoNavigation:
         default:
@@ -594,7 +630,7 @@ ViewContainer* ViewManager::createContainer(const Profile::Ptr info)
 
     connect( container, SIGNAL(newViewRequest()), this, SIGNAL(newViewRequest()) );
     connect( container, SIGNAL(moveViewRequest(int,int,bool&)), 
-	this , SLOT(containerMoveViewRequest(int,int,bool&)) );
+    this , SLOT(containerMoveViewRequest(int,int,bool&)) );
     connect( container , SIGNAL(viewRemoved(QWidget*)) , this , SLOT(viewCloseRequest(QWidget*)) );
     connect( container , SIGNAL(closeRequest(QWidget*)) , this , SLOT(viewCloseRequest(QWidget*)) );
     connect( container , SIGNAL(activeViewChanged(QWidget*)) , this , SLOT(viewActivated(QWidget*)));
@@ -603,14 +639,14 @@ ViewContainer* ViewManager::createContainer(const Profile::Ptr info)
 }
 void ViewManager::containerMoveViewRequest(int index, int id, bool& moved)
 {
-	ViewContainer* container = qobject_cast<ViewContainer*>(sender());
-	SessionController* controller = qobject_cast<SessionController*>(ViewProperties::propertiesById(id));
+    ViewContainer* container = qobject_cast<ViewContainer*>(sender());
+    SessionController* controller = qobject_cast<SessionController*>(ViewProperties::propertiesById(id));
 
-	if (!controller)
-		return;
+    if (!controller)
+        return;
 
-	createView(controller->session(),container,index);
-	moved = true;
+    createView(controller->session(),container,index);
+    moved = true;
 }
 void ViewManager::setNavigationMethod(NavigationMethod method)
 {
@@ -620,6 +656,15 @@ void ViewManager::setNavigationMethod(NavigationMethod method)
 
     if ( collection )
     {
+        // FIXME: The following disables certain actions for the KPart that it
+        // doesn't actually have a use for, to avoid polluting the action/shortcut
+        // namespace of an application using the KPart (otherwise, a shortcut may
+        // be in use twice, and the user gets to see an "ambiguous shortcut over-
+        // load" error dialog). However, this approach sucks - it's the inverse of
+        // what it should be. Rather than disabling actions not used by the KPart,
+        // a method should be devised to only enable those that are used, perhaps
+        // by using a separate action collection.
+
         QAction* action;
 
         action = collection->action( "next-view" );
@@ -636,6 +681,12 @@ void ViewManager::setNavigationMethod(NavigationMethod method)
 
         action = collection->action( "rename-session" );
         if ( action ) action->setEnabled( _navigationMethod != NoNavigation );
+
+        action = collection->action( "move-view-left" );
+        if ( action ) action->setEnabled( _navigationMethod != NoNavigation );
+
+        action = collection->action( "move-view-right" );
+        if ( action ) action->setEnabled( _navigationMethod != NoNavigation );
     }
 }
 
@@ -643,7 +694,7 @@ ViewManager::NavigationMethod ViewManager::navigationMethod() const { return _na
 
 void ViewManager::containerViewsChanged(QObject* container)
 {
-    if ( container == _viewSplitter->activeContainer() )
+    if (_viewSplitter && container == _viewSplitter->activeContainer() )
     {
         emit viewPropertiesChanged( viewProperties() );
     } 
@@ -667,9 +718,11 @@ void ViewManager::viewCloseRequest(QWidget* view)
         if ( session->views().count() == 0 )
             session->close();
     }
-        
-    focusActiveView();
-	updateDetachViewState();
+    //we only update the focus if the splitter is still alive
+    if (_viewSplitter) {
+        focusActiveView();
+        updateDetachViewState();
+    }
 }
 
 TerminalDisplay* ViewManager::createTerminalDisplay(Session* session)
@@ -769,6 +822,9 @@ void ViewManager::applyProfile(TerminalDisplay* view , const Profile::Ptr info,
     bool blinkingCursor = info->property<bool>(Profile::BlinkingCursorEnabled);
     view->setBlinkingCursor(blinkingCursor);  
 
+    bool blinkingText = info->property<bool>(Profile::BlinkingTextEnabled);
+    view->setBlinkingTextEnabled(blinkingText);
+
     bool bidiEnabled = info->property<bool>(Profile::BidiRenderingEnabled);
     view->setBidiEnabled(bidiEnabled);
 
@@ -811,8 +867,8 @@ void ViewManager::profileChanged(Profile::Ptr profile)
 
         // if session uses this profile, update the display
         if ( iter.key() != 0 && 
-			 iter.value() != 0 && 
-			 SessionManager::instance()->sessionProfile(iter.value()) == profile ) 
+             iter.value() != 0 && 
+             SessionManager::instance()->sessionProfile(iter.value()) == profile ) 
         {
             applyProfile(iter.key(),profile,true);
         }
@@ -838,9 +894,154 @@ QList<ViewProperties*> ViewManager::viewProperties() const
     return list;
 }
 
+void ViewManager::saveSessions(KConfigGroup& group)
+{
+    // find all unique session restore IDs
+    QList<int> ids;
+    QHash<Session*,int> unique;
+
+    // first: sessions in the active container, preserving the order
+    ViewContainer* container = _viewSplitter->activeContainer();
+    Q_ASSERT(container);
+    TerminalDisplay* activeview = dynamic_cast<TerminalDisplay*>(container->activeView());
+
+    QListIterator<QWidget*> viewIter(container->views());
+    int tab = 1;
+    while (viewIter.hasNext())
+    {
+        TerminalDisplay *view = dynamic_cast<TerminalDisplay*>(viewIter.next());
+        Q_ASSERT(view);
+        Session *session = _sessionMap[view];
+        ids << SessionManager::instance()->getRestoreId(session);
+        if (view == activeview) group.writeEntry("Active", tab);
+        unique.insert(session, 1);
+        tab++;
+    }
+
+    // second: all other sessions, in random order
+    // we don't want to have sessions restored that are not connected
+    foreach(Session* session, _sessionMap)
+        if (!unique.contains(session))
+        {
+            ids << SessionManager::instance()->getRestoreId(session);
+            unique.insert(session, 1);
+        }
+
+    group.writeEntry("Sessions", ids);
+}
+
+void ViewManager::restoreSessions(const KConfigGroup& group)
+{
+    QList<int> ids = group.readEntry("Sessions", QList<int>());
+    int activeTab  = group.readEntry("Active", 0);
+    TerminalDisplay *display = 0;
+
+    int tab = 1;
+    foreach(int id, ids)
+    {
+        Session *session = SessionManager::instance()->idToSession(id);
+        createView(session);
+        if (!session->isRunning())
+            session->run();
+        if (tab++ == activeTab)
+            display = dynamic_cast<TerminalDisplay*>(activeView());
+    }
+
+    if (display)
+    {
+        _viewSplitter->activeContainer()->setActiveView(display);
+        display->setFocus(Qt::OtherFocusReason);
+    }
+}
+
 uint qHash(QPointer<TerminalDisplay> display)
 {
     return qHash((TerminalDisplay*)display);
 }
 
+int ViewManager::sessionCount()
+{
+    return this->_sessionMap.size();
+}
+
+int ViewManager::currentSession()
+{
+    QHash<TerminalDisplay*,Session*>::iterator i;
+    for (i = this->_sessionMap.begin(); i != this->_sessionMap.end(); ++i)
+        if (i.key()->isVisible())
+            return i.value()->sessionId();
+    return -1;
+}
+
+int ViewManager::newSession()
+{
+    Profile::Ptr profile = profile = SessionManager::instance()->defaultProfile();
+    Session* session = SessionManager::instance()->createSession(profile);
+
+    this->createView(session);
+    session->run();
+
+    return session->sessionId();
+}
+
+int ViewManager::newSession(QString profile, QString directory)
+{
+    QList<Profile::Ptr> profilelist = SessionManager::instance()->loadedProfiles();
+    QList<Profile::Ptr>::iterator i = profilelist.begin();
+
+    Profile::Ptr profileptr = SessionManager::instance()->defaultProfile();
+
+    while (i != profilelist.end() )
+    {
+        Profile::Ptr ptr = *i;
+        if ( ptr->name().compare(profile) == 0)
+            profileptr = ptr;
+        i++;
+    }
+
+    Session* session = SessionManager::instance()->createSession(profileptr);
+    session->setInitialWorkingDirectory(directory);
+
+    this->createView(session);
+    session->run();
+
+    return session->sessionId();
+}
+
+QStringList ViewManager::profileList()
+{
+    QList<Profile::Ptr> profilelist = SessionManager::instance()->loadedProfiles();
+    QList<Profile::Ptr>::iterator i = profilelist.begin();
+    QStringList list;
+    while (i != profilelist.end() )
+    {
+        Profile::Ptr ptr = *i;
+        list.push_back(ptr->name());
+        i++;
+    }
+
+    return list;
+}
+
+void ViewManager::nextSession()
+{
+    this->nextView();
+}
+
+void ViewManager::prevSession()
+{
+    this->previousView();
+}
+
+void ViewManager::moveSessionLeft()
+{
+    this->moveActiveViewLeft();
+}
+
+void ViewManager::moveSessionRight()
+{
+    this->moveActiveViewRight();
+}
+
 #include "ViewManager.moc"
+
